#include "fintrf.h"
!
		SUBROUTINE MEXFUNCTION(NLHS, PLHS, NRHS, PRHS)
!
!     This is the gateway subroutine for LSQSPARSE mex function
!
		USE SPARSEREG
		IMPLICIT NONE
!
!     MEXFUNCTION ARGUMENTS
!
		MWPOINTER :: PLHS(*), PRHS(*)
		INTEGER :: NLHS, NRHS
!
!     FUNCTION DECLARATIONS
!
		INTEGER :: MXISCHAR,MXISLOGICAL,MXISNUMERIC
		INTEGER*4 :: MEXPRINTF
		MWPOINTER :: MXCREATEDOUBLEMATRIX,MXGETPR,MXGETSTRING
		MWSIZE :: MXGETM, MXGETN
!
!     SOME LOCAL VARIABLES
!		
      INTEGER :: MAXITERS,STATUS
      MWSIZE :: I,N,N1,N2,PENNAMELEN,PENPARAMS
      CHARACTER(LEN=10) :: PENTYPE
      REAL(KIND=DBLE_PREC), ALLOCATABLE, DIMENSION(:) :: A,B,MAXLAMBDA,PENPARAM
!
!     CHECK FOR INPUT/OUTPUT ARGUMENT TYPES
!
      IF (NRHS.NE.4) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:lsqmaxlambda:nInput','Four input requried.')
      ELSEIF (NLHS>1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:lsqmaxlambda:nOutput','At most one output requried.')
      ELSEIF (MXISNUMERIC(PRHS(1))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:lsqmaxlambda:Input1','Input 1 must be a numerical array.')
      ELSEIF (MXISNUMERIC(PRHS(2))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:lsqmaxlambda:Input2','Input 2 must be a scalar.')
      ELSEIF (MXISCHAR(PRHS(3))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:lsqmaxlambda:Input3','Input 3 must be a string.')
      ELSEIF (MXISNUMERIC(PRHS(4))/=1) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:lsqmaxlambda:Input4','Input 4 must be a numerical array.')
      END IF
!
!     PREPARE INPUTS FOR COMPUTATIONAL ROUTINE
!
      N1 = MXGETM(PRHS(1))
      N2 = MXGETN(PRHS(1)) 
      N = N1*N2
      ALLOCATE(A(N),B(N),MAXLAMBDA(N))
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(1)),A,N)
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(2)),B,N)
      PENNAMELEN = MXGETM(PRHS(3))*MXGETN(PRHS(3))
      STATUS = MXGETSTRING(PRHS(3), PENTYPE, PENNAMELEN)
      IF (STATUS/=0) THEN
         CALL MEXERRMSGIDANDTXT('MATLAB:lsqmaxlambda:readError','Error reading string.')
      END IF
      PENPARAMS = MXGETM(PRHS(4))*MXGETN(PRHS(4))
      ALLOCATE(PENPARAM(PENPARAMS))
      CALL MXCOPYPTRTOREAL8(MXGETPR(PRHS(4)),PENPARAM,PENPARAMS)      
!
!     CALL THE COMPUTATION ROUTINE AND COPY RESULT TO MATLAB ARRAYS
!
      DO I=1,N
         MAXLAMBDA(I) = MAX_RHO(A(I),B(I),PENTYPE(1:PENNAMELEN),PENPARAM)
      END DO
      PLHS(1) = MXCREATEDOUBLEMATRIX(N1,N2,0)
      CALL MXCOPYREAL8TOPTR(MAXLAMBDA,MXGETPR(PLHS(1)),N)
!
!     FREE MEMORY
!
      DEALLOCATE (A,B,MAXLAMBDA,PENPARAM)
      RETURN
		END SUBROUTINE MEXFUNCTION

