%% Benchmark data example 2 - Brain Tumor Data - Section 5.2
% Estimate a generalized lasso model (sparse fused lasso) via the constrained 
% lasso  
 
%% Initial Setup 

% clear workspace
clear; 

%# Load & organize data #%
% load R output
load('data2.mat')
% extract dimensions
n = size(y, 1);
p = n;  % signal approximation so X = I, p = n
X = eye(n);



%% Estimate using constrained lasso solution path algorithm

%# Model setup #%
% create penalty matrix 
D = [eye(p-1) zeros(p-1, 1)] - [zeros(p-1, 1) eye(p-1)];
% number of rows in D
m = size(D, 1);


%# Transform to constrained lasso #%
% calculate SVD
[U, Sigma, V] = svd(D);
% extract singular values
singVals = diag(Sigma);
% determine rank
rankD = nnz(singVals > abs(singVals(1))*eps(singVals(1))*max(size(D)));
% extract submatrices of V & U
V1 = V(:, 1:rankD); V2 = V(:, rankD+1:end); 
U1 = U(:, 1:rankD); U2 = U(:, rankD+1:end);

% calcuate Moore-Penrose inverse of D
Dplus = V1*bsxfun(@times, U1', 1./singVals);
% transform design matrix 
XDplus = X*Dplus;


%# transform to "tilde" form (to get rid of gamma)
XV2 = X*V2;
% projection onto C(XV2)
Pxv2 = XV2*inv(XV2'*XV2)*XV2';
% orthogonal projection matrix
Mxv2 = eye(size(XV2, 1)) - Pxv2;
% create "tilde" data 
ytilde = Mxv2*y;
Xtilde = Mxv2*XDplus;
 

%# Constrained lasso solution path #%
% estimate solution path in terms of alpha
[rhosPath, alphaHatEstimatesPath] = lsq_classopath(Xtilde, ytilde, [], [], ...
    [], [], 'qp_solver', 'matlab', 'init_method', 'qp');
% transform back to beta  
betaHatEstimatesPath = (eye(size(V2, 1)) - ...
    V2*((XV2'*XV2)\XV2'*X))*Dplus*alphaHatEstimatesPath + ...
    repmat(V2*((XV2'*XV2)\XV2'*y), [1 size(rhosPath, 2)]);

%# plot solution paths #%
% constrained lasso solution path
figure;
plot(rhosPath(1:end-1), betaHatEstimatesPath(:, 1:end-1));
title({'Data Example 2: Brain Tumor Data', ...
    'Solution Path: betaHat (classopath)'})
xlabel('rho');
ylabel('beta(rho)');
xlim([min(rhosPath),max(rhosPath)]);
 
% plot generalized lasso solution path (from genlasso R package)
figure;
plot(lambdaPath, betaHatPathFused);
title({'Data Example 2: Brain Tumor Data',...
    'Solution Path: betaHat (genlasso)'})
xlabel('lambda');
ylabel('beta(lambda)');
xlim([min(lambdaPath), max(lambdaPath)]);

% save output to file to graph in R
save('data2classo.mat', 'y', 'betaHatPathFused', 'rhosPath', ...
    'betaHatEstimatesPath', 'lambdaPath')
