### Benchmark data example 2 - Brain Tumor Data - Section 5.2
## Estimate a sparse fused lasso model (using the generalized lasso) on the 
## brain tumor data

### Initial Setup ###
# clean up
rm(list=ls())

## load packages
# install function for packages    
packages <- function(x){
  x <- as.character(match.call()[[2]])
  if (!require(x, character.only=TRUE)){
    install.packages(pkgs=x, repos="http://cran.r-project.org")
    require(x, character.only=TRUE)
  }
}
# generalized lasso package
packages(genlasso)
# functions to read/save MATLAB data
packages(R.matlab)
# package with cgh data
packages(cghFLasso)


### Load & organize data ###
data(CGH)
y = CGH$GBM.y
n = length(y)


### Model estimation ###
# fit regular fused lasso
flFit = fusedlasso1d(y)

## extract model output
# tuning parameter grid
lambdaPath = flFit$lambda
# solution path for fused lasso
betaHatPathFused = flFit$beta
# soft threshold for sparse fused lasso estimates
betaHatPathSparseFL = softthresh(flFit, lambda = lambdaPath, gamma = 0.01)
# extract generalized lasso penalty matrix D
D = genlasso::getD1d(n)

# write estimates to .mat file for use in Matlab
writeMat(con = "data2.mat", y = y, lambdaPath = lambdaPath, 
         betaHatPathFused = betaHatPathFused, DfromR = D, 
         betaHatPathSparseFL = betaHatPathSparseFL)




