### Benchmark data example 2 - Brain Tumor Data - Section 5.2
## Graph estimates from genlasso and classopath

### Initial Setup ###
# clean up
rm(list=ls())

## load packages
# install function for packages    
packages <- function(x){
  x <- as.character(match.call()[[2]])
  if (!require(x, character.only=TRUE)){
    install.packages(pkgs=x, repos="http://cran.r-project.org")
    require(x, character.only=TRUE)
  }
}
# functions to read/save MATLAB data
packages(R.matlab)
packages(reshape2)
packages(ggplot2)


### Load & organize data ###
data = readMat(con = 'data2classo.mat', fixNames = TRUE)

# extract output 
y = data$y
lambdaPath = data$lambdaPath
betaHatPathFused = data$betaHatPathFused
rhosPath = data$rhosPath
betaHatEstimatesPath = data$betaHatEstimatesPath
n = length(y)

# put data together for graphing
dataGraph = data.frame(y = y, Classopath = betaHatEstimatesPath[, 26], 
                       Genlasso = betaHatPathFused[, 26])


### Create graph ###
# melt data for graphing
ggplot(data = dataGraph, aes(x= 1:n, y = y)) + 
  geom_point(color="grey55", size = 4) + theme_bw() + 
  ylab(bquote(paste(log[2], " Ratio"))) + 
  xlab("Genome Order") + 
  geom_point(data = dataGraph, 
             aes(x = 1:n, y = Classopath, color = "Classo"), 
             size = 3, shape = 4) + 
  geom_line(data = dataGraph, aes(x = 1:n, y = Genlasso, color = "Genlasso"),
            size = 1) + geom_hline(yintercept=0) + 
  theme(panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(), 
        axis.title.y = element_text(size = rel(1.11), angle = 90), 
        axis.title.x = element_text(size = rel(1.11)),
        axis.text.x = element_text(size = rel(1.11)), 
        axis.text.y = element_text(size = rel(1.11)), 
        legend.position = c(0.77, 0.90), legend.text=element_text(size=11), 
        legend.background = element_rect(fill = "transparent")) + 
  scale_x_continuous(breaks=seq(0, 1000, 200)) + 
  scale_color_manual("", values = c("Genlasso" = "black", "Classo" = "red"),
                     labels = list("Constrained Lasso", "Generalized Lasso")) +   
  guides(colour = guide_legend(override.aes = list(linetype = c(0,1), 
                                                   shape = c(4, NA))))