### Benchmark data example 3 - Microbiome Data - Section 5.3
## Graph estimates from zero-sum regression and classopath


### Initial Setup ###
# clean up
rm(list=ls())

## load packages
# install function for packages 
packages <- function(x){
  x <- as.character(match.call()[[2]])
  if (!require(x, character.only=TRUE)){
    install.packages(pkgs=x, repos="http://cran.r-project.org")
    require(x, character.only=TRUE)
  }
}
# functions to read/save MATLAB data
packages(R.matlab)
packages(ggplot2)
packages(dplyr)
packages(reshape2)

### Load & organize data ###
dataEstimates = readMat(con = 'data3both.mat', fixNames = TRUE)
# extract output 
rhosPath = as.numeric(dataEstimates$rhosPath)
betaHatPathClasso = dataEstimates$betaHatPathClasso
betaHatPathZeroSum = dataEstimates$betaHatPathZeroSum
ell1NormPath = dataEstimates$ell1NormPath
predR2 = dataEstimates$predR2
fittedClasso = dataEstimates$fittedClasso

### Plot solutions paths ###
# function to plot paths
plotSolPath <- function(betaPath, rhosPath){
  
  ### organize data for plotting
  # add values of rho as column names
  colnames(betaPath) = rhosPath
  # melt data to be ggplot friendly
  betaPathGraph = melt(t(betaPath))
  # rename columns
  colnames(betaPathGraph) = c("rho", "coef", "value")
  
  ggplot(betaPathGraph, aes(x = rho, y = value, group = coef, color = coef)) + 
    geom_line() + theme_bw() + 
    xlab(expression(paste("||", hat(beta)(rho),"||"[1]))) + 
    ylab(expression(hat(beta)(rho))) + 
    theme(axis.title.y = element_text(size = rel(1.69)),
          axis.title.x = element_text(size = rel(1.69)),
          axis.text.x = element_text(size = rel(1.69)),
          axis.text.y = element_text(size = rel(1.69)),
          panel.grid.major.x = element_blank(),
          panel.grid.minor.x = element_blank(), 
          legend.position="none") + 
    ylim(-1.8, 1.1)  + scale_x_continuous(trans = "reverse")
}

# plot constrained lasso solution path
plotSolPath(betaHatPathClasso, ell1NormPath)
# plot zero-sum regression solution path
plotSolPath(betaHatPathZeroSum, ell1NormPath)



### Plot observed vs. fitted values ###
# function to plot observed vs fitted values 
plotFitted <- function(dat, Rsqd){
  ggplot(dat, aes(x = fitted, y = y)) + geom_point(size=2, shape=1, color="steelblue4") + theme_bw() + 
    geom_abline(intercept = 0, slope = 1, linetype="dashed") + 
    xlab("Fitted Values") + 
    ylab("Observed Values") + 
    theme(axis.title.y = element_text(size = rel(1.69)),
          axis.title.x = element_text(size = rel(1.69)),
          axis.text.x = element_text(size = rel(1.87), color="black"),
          axis.text.y = element_text(size = rel(1.87), color="black"),
          legend.position="none") +
    annotate("text", label = paste("Predicted~R^{2}==", round(Rsqd, 3)), parse=TRUE, x = 1.17, y = -6.126, size=6) 
}

# calculate fitted values
observed <- dataEstimates$y
df_fitted <- data.frame(fittedClasso, observed)
colnames(df_fitted) <- c("fitted", "y")

# graph observed vs. fitted values
plotFitted(df_fitted, predR2)